module Dialogs where

import Graphics.UI.WX
import Graphics.UI.WXCore
import HXML
import Char

matchDialog :: Window w -> IO (Maybe (String, String))
matchDialog w = do {
		wfunc <- dialog w [text := "Function:   match"];
		p <- panel wfunc [];
		
		--Elements
		labvar <- staticText p [text:="Var:"];
		labexp <- staticText p [text:="RegExp:"];
		var <- textEntry p [text:=""];
		exp <- textEntry p [text:=""];
		quit <- button p [text:="OK"];
		cancF <- button p [text:="Canc"];
		
		--Layout
		set wfunc [layout := container p (margin 5 $ fill $
	    				column 5 [
	    					fill $ row 5 [widget labvar, hspace 5, hfill $ widget var],
	    					fill $ row 5 [widget labexp, widget exp],
	    					floatLeft $ row 5 [widget quit, widget cancF]
	    					]
	    			)];
	    			
	    	showModal wfunc (\stop -> do {
	    			set quit [on command := do {
	    					v <- get var text;
	    					e <- get exp text;
	    					if ((v == "") || (e == ""))
	    					then stop (Nothing)
	    					else stop (Just (v, e))
						}];
				set cancF [on command := stop (Nothing)];
					})
	    	}

funcDialog :: Window a -> String -> IO (Maybe (String,String))
funcDialog w f = do {
		wfunc <- dialog w [text := "Function:   "++f];
	     	p <- panel wfunc [];
	     	
	     	--Elements	
	     	explain <- staticText p [text:="Insert for each argument a variable a number or a function"];
	     	labarg1 <- staticText p [text:="Arg1: "];
	     	labarg2 <- staticText p [text:="Arg2: "];
	     	r1 <- textEntry p [text:=""];
	     	r2 <- textEntry p [text:=""];
	     	add1 <- comboBox p [items := ["Add Function", "append", "+", "-", "*", "/"], style:=wxCB_READONLY, selection:=0];
	     	add2 <- comboBox p [items := ["Add Function", "append", "+", "-", "*", "/"], style:=wxCB_READONLY, selection:=0];
	     	setF <- button p [text:="OK"];
	     	cancF <- button p [text:="Canc"];
	     	
	    	--Layout
	    	set wfunc [layout := container p (margin 5 $ fill $
	    				column 5 [
	    					floatLeft $ widget explain,
	    					hfill $ row 5 [widget labarg1, hfill $ widget r1, widget add1],
	    					hfill $ row 5 [widget labarg2, hfill $ widget r2, widget add2],
	    					floatLeft $ row 5 [widget setF, widget cancF]
	    					]
	    			)];
	    	set add1 [on select := do {
	    			i <- get add1 selection;
	    			func <- get add1 (item i);
	    			case i of
	    			0 -> return();
	    			_ -> do {
					y <- funcDialog wfunc func;
					case y of
						Just (arg1, arg2) -> do {set r1 [text:=arg1++" "++func++" "++arg2];}
						Nothing -> return(); }
				}];
				
		set add2 [on select := do {
	    			i <- get add2 selection;
	    			func <- get add2 (item i);
	    			case i of
	    			0 -> return();
	    			_ -> do {
					y <- funcDialog wfunc func;
					case y of
						Just (arg1, arg2) -> do {set r2 [text:=arg1++" "++func++" "++arg2];}
						Nothing -> return(); }
				}];
	    			
	    	showModal wfunc (\stop -> do {
	    			set setF [on command := do {
	    					arg1 <- get r1 text;
	    					arg2 <- get r2 text;
	    					if ((arg1 == "") || (arg2 == ""))
	    					then stop (Nothing)
	    					else stop (Just (arg1,arg2))
						}];
				set cancF [on command := stop (Nothing)];
					})
	    	}

termDialog :: Window a -> [String] -> [String] -> [String] -> String -> IO (Maybe (Int, [Tree XMLNode],[String],[String]))
termDialog w tTags pattTags pattVars node = do{
	numvar<-varCreate [];
	varSet numvar "0";
	sons<-varCreate [];
	varSet sons [];
	pTags<-varCreate [];
	varSet pTags [];
	pVars<-varCreate [];
	varSet pVars [];
	
     	wsub <- dialog w [text:="Node insertion: "++node];
     	psub <- panel wsub [];
     	
     	--Elements	
     	whereToInsert <- singleListBox psub [tooltip:="Select the tag in which you want to insert the new node"];
     	addItems whereToInsert pattTags;
     	textTags<-singleListBox psub [tooltip:="Select the tag(s) you want to insert", enabled:=False];
     	addItems textTags tTags;
     	bool <- singleListBox psub [items:=["and","or","xor"],tooltip:="Select if you want to insert a boolean operator", enabled:=False];
     	ren <- radioBox psub Vertical ["Without renaming","With renaming"] [tooltip:="Select the search mode for the selected tag",enabled:=False];
     	insertTag <- button psub [text:=">>",enabled:=False];
     	insertText <- button psub [text:="Text >>",enabled:=False];
     	insertVar <- button psub [text:="Var >>",enabled:=False];
     	view <- singleListBox psub [tooltip:="Displays the inserted nodes"];
     	ok <- button psub [text:="OK", enabled:=False];
     	quit <- button psub [text:="Cancel"];
     	
     	--Layout
    	set wsub [layout := container psub (margin 5 $ fill $
    				row 5 [
    					fill $ boxed "Where to insert" $ fill $ widget whereToInsert,
    					column 3 [fill $ boxed "What to insert" $ fill $ widget textTags, fill $ widget bool],
    					column 5 [fill $ widget ren, widget insertTag, widget insertText, widget insertVar],
    					column 5 [fill $ widget view, widget ok, widget quit]
    				      ]
    			)];
    	windowSetClientSize wsub (sz 500 250);
    	
    	set whereToInsert [on select := do {
    			set textTags[enabled:=True];
    			set bool[enabled:=True];
    			set insertText[enabled:=True];
    			v<-varGet numvar;
    			i<- get whereToInsert selection;
    			tag <- get whereToInsert (item i);
    			if ((v=="1") || (tag == "Root"))
    			then set insertVar[enabled:=False]
    			else set insertVar[enabled:=True]
    			}];
    	
    	set textTags [on select := do {
    			set ren[enabled:=True];
    			set insertTag[enabled:=True];
    			}];
    	
    	set bool [on select := do {	
    			i <- get bool selection;
    			b <- get bool (item i);
    			bv <- varGet pVars;
    			h <- get whereToInsert selection;
    			tag <- get whereToInsert (item h);
    			k <- return(controlNumVar (length bv) tag);
    			y <- simpleTermDialog wsub tTags pattVars k;
    			case y of
				Just (newSons,newTags,newVars) -> do {
					varSet pVars (newVars++bv);
					j <- return(controlNumVar (length newVars) tag);
					pt <- varGet pTags;
					varSet pTags (pt++newTags);
					figli <- varGet sons;
					varSet sons (figli++[(Tree (ELNode b []) newSons)]);
					itemAppend view (b++" ...");
					if (j==0)
					then do {
						set insertVar[enabled:=False];
						varSet numvar "1";
					}
					else return()
				}
				Nothing -> return();
    			}];
    			
    	set insertTag [on command := do {
    			i <- get textTags selection;
    			son <- get textTags (item i);
    			j <- get ren selection;
    			r <- get ren (item j);
    			ts <- varGet pTags;
    			varSet pTags (ts++[son]);
    			(sw,st) <- return(controlSon son r);
    			itemAppend view (sw);
    			set ok[enabled:=True];
    			if (node == "Single Node")
			then do {
				varSet sons [st];
				set textTags[enabled:=False];
				set bool[enabled:=False];
				set insertTag[enabled:=False];
				set insertText[enabled:=False];
			    	set insertVar[enabled:=False];
			    	set whereToInsert[enabled:=False];
			    	set ren[enabled:=False];	
			}
			else do {
				figli <- varGet sons;
				varSet sons (figli++[st]);
			}
    		}];
    	
    	set insertText [on command := do {
    			y <- textVarDialog wsub "Text";
    			case y of
				Just t -> do {
					itemAppend view ('\'':(t++"'"));
					set ok[enabled:=True];
					if (node == "Single Node")
					then do {
						varSet sons [(Tree (TXNode t) [])];
						set textTags[enabled:=False];
			    			set bool[enabled:=False];
			    			set insertText[enabled:=False];
			    			set insertVar[enabled:=False];
			    			set whereToInsert[enabled:=False];
			    			set ren[enabled:=False];	
					}
					else do {
						figli <- varGet sons;
						varSet sons (figli++[(Tree (TXNode t) [])]);
					}
				}
				Nothing -> return();
    			}];
    	
    	set insertVar [on command := do {
    			y <- textVarDialog wsub "Var";
    			case y of
				Just t -> do {
					okay <- return(controlVars pattVars t);
					if (okay)
					then do {
						itemAppend view t;
						v <- varGet pVars;
						varSet pVars (t:v);
						varSet numvar "1";
						set ok[enabled:=True];
						set insertVar[enabled:=False];
						if (node == "Single Node")
						then do {
							set textTags[enabled:=False];
				    			set bool[enabled:=False];
				    			set insertText[enabled:=False];
				    			set insertVar[enabled:=False];
				    			set whereToInsert[enabled:=False];
				    			set ren[enabled:=False];	
						}
						else return()
					}else warningDialog wsub "Attention" "The variable inserted is not valid"
				}
				Nothing -> return();
    			}];
    	
    	
    	showModal wsub (\stop -> do {
	    			set ok [on command := do {
	    					i1 <- get whereToInsert selection;
	    					i2 <- varGet sons;
	    					vars <- varGet pVars;
	    					tags <- varGet pTags;
	    					if ((node /= "Single Node") && (((length i2)+(length vars)) == 1))
	    					then warningDialog wsub "Attention" "You have inserted only one son under a boolean node"
	    					else stop (Just ((i1+1),i2,tags,vars))
						}];
				set quit [on command := stop (Nothing)];
					})
	    	}

simpleTermDialog :: Window a -> [String] -> [String] -> Int -> IO (Maybe ([Tree XMLNode],[String],[String]))
simpleTermDialog w tTags pattVars enableVar = do{
	sons<-varCreate [];
	varSet sons [];
	pTags<-varCreate [];
	varSet pTags [];
	pVars<-varCreate [];
	varSet pVars [];
	
     	wsub <- dialog w [text:="Node insertion: And/Or/Xor"];
     	psub <- panel wsub [];
     	
     	--Elements	
     	textTags<-singleListBox psub [tooltip:="Select the tag(s) you want to insert"];
     	addItems textTags tTags;
     	bool <- singleListBox psub [items:=["and","or","xor"],tooltip:="Select if you want to insert a boolean operator"];
     	ren <- radioBox psub Vertical ["Without renaming","With renaming"] [tooltip:="Select the search mode for the selected tag",enabled:=False];
     	insertTag <- button psub [text:=">>",enabled:=False];
     	insertText <- button psub [text:="Text >>"];
     	insertVar <- button psub [text:="Var >>"];
     	verifyButtonVar insertVar enableVar;
     	view <- singleListBox psub [tooltip:="Displays the inserted nodes"];
     	ok <- button psub [text:="OK", enabled:=False];
     	quit <- button psub [text:="Cancel"];
     	
     	--Layout
    	set wsub [layout := container psub (margin 5 $ fill $
    				row 5 [
    					column 3 [fill $ boxed "What to insert" $ fill $ widget textTags, fill $ widget bool],
    					column 5 [fill $ widget ren, widget insertTag, widget insertText, widget insertVar],
    					column 5 [fill $ widget view, widget ok, widget quit]
    				      ]
    			)];
    	windowSetClientSize wsub (sz 500 250);
    	
    	set textTags [on select := do {
    			set ren[enabled:=True];
    			set insertTag[enabled:=True];
    			}];
    	
    	set bool [on select := do {	
    			i <- get bool selection;
    			b <- get bool (item i);
    			bv <- varGet pVars;
    			k <- return(controlNumVar2 enableVar (length bv));
    			y <- simpleTermDialog wsub tTags pattVars k;
    			case y of
				Just (newSons,newTags,newVars) -> do {
					varSet pVars (newVars++bv);
					j <- return(controlNumVar2 enableVar (length newVars));
					pt <- varGet pTags;
					varSet pTags (pt++newTags);
					figli <- varGet sons;
					varSet sons (figli++[(Tree (ELNode b []) newSons)]);
					itemAppend view (b++" ...");
					if (j==0)
					then set insertVar[enabled:=False]
					else return()
				}
				Nothing -> return();
    			}];
    			
    	set insertTag [on command := do {
    			i <- get textTags selection;
    			son <- get textTags (item i);
    			j <- get ren selection;
    			r <- get ren (item j);
    			ts <- varGet pTags;
    			varSet pTags (ts++[son]);
    			(sw,st) <- return(controlSon son r);
    			itemAppend view (sw);
    			set ok[enabled:=True];
    			figli <- varGet sons;
			varSet sons (figli++[st]);
    		}];
    	
    	set insertText [on command := do {
    			y <- textVarDialog wsub "Text";
    			case y of
				Just t -> do {
					itemAppend view ('\'':(t++"'"));
					set ok[enabled:=True];
					figli <- varGet sons;
					varSet sons (figli++[(Tree (TXNode t) [])]);
				}
				Nothing -> return();
    			}];
    	
    	set insertVar [on command := do {
    			y <- textVarDialog wsub "Var";
    			case y of
				Just t -> do {
					okay <- return(controlVars pattVars t);
					if (okay)
					then do {
						itemAppend view t;
						v <- varGet pVars;
						varSet pVars (t:v);
						set ok[enabled:=True];
						set insertVar[enabled:=False];
					}else warningDialog wsub "Attention" "The variable inserted is not valid"
				}
				Nothing -> return();
    			}];
    	
    	
    	showModal wsub (\stop -> do {
	    			set ok [on command := do {
	    					i2 <- varGet sons;
	    					vars <- varGet pVars;
	    					tags <- varGet pTags;
	    					if (((length i2)+(length vars)) == 1)
	    					then warningDialog wsub "Attention" "You have inserted only one son under a boolean node"
	    					else stop (Just (i2,tags,vars))
						}];
				set quit [on command := stop (Nothing)];
					})
	    	}

textVarDialog::Window a -> String -> IO(Maybe String)
textVarDialog w t = do {
	wtext <- dialog w [text:="Add "++t];
     	ptext <- panel wtext [];
     	
     	--Elements
     	lab <- staticText ptext [text:="Enter "++t];
     	tx <- textEntry ptext [];
     	ok <- button ptext [text:="OK"];
     	quit <- button ptext [text:="Cancel"];
     	
     	--Layout
     	set wtext [layout := container ptext ( margin 5 $ fill $ column 5 [
     						floatLeft $ row 5 [widget lab, fill $ widget tx],
     						floatLeft $ row 5 [widget ok, widget quit]     						
     						]
     		)];
     	windowSetClientSize wtext (sz 240 90);
     	
     	showModal wtext (\stop -> do {
	    			set ok [on command := do {
	    					i <- get tx text;
	    					if (i=="")
	    					then stop (Nothing)
	    					else stop (Just i)
						}];
				set quit [on command := stop (Nothing)];
					})
}

nestingDialog :: Window a -> IO (Maybe String)
nestingDialog w = do {
	wnes <- dialog w [text:="Nesting query"];
     	pnes <- panel wnes [];
	
	--Elements
	loadq <- button pnes [text:="Load Query"];
        selectQuery <- singleListBox pnes [];
        insert <- button pnes [text:="OK", enabled:=False];
        quit <- button pnes [text:="Cancel"];
        
        --Layout
        set wnes [layout := container pnes ( margin 5 $ fill $ column 5 [
       						floatLeft $ widget loadq,
       						floatLeft $ fill $ boxed "Queries" $ fill $ widget selectQuery,
       						floatLeft $ row 10 [widget insert, widget quit]
       						]
       		)];
       	windowSetClientSize wnes (sz 400 200);
       	
       	set selectQuery [on select := set insert[enabled:=True]];
       	
       	set loadq [on command:= do {
       			file<-fileOpenDialog wnes False True "Query File" [("Query file (*.phil)",["*.phil"])] "" "";
			case file of
				Nothing -> return();
				Just filename -> do {content<-readFile filename;
						     itemsDelete selectQuery;
						     extractQueryfromFile selectQuery content [];
						    }
       		}];
       	
       	showModal wnes (\stop -> do {
	    			set insert [on command := do {
	    					i <- get selectQuery selection;
	    					query<-get selectQuery (item i);
	    					query<-return(tail(tail(tail(tail(tail(tail(tail(query))))))));
	    					stop (Just query)
						}];
				set quit [on command := stop (Nothing)];
					})
}

varTextDialog :: Window a -> String -> IO (Maybe (String,Int,String))
varTextDialog w t = do{
     	wsub <- dialog w [text:="Add New "++t];
     	psub <- panel wsub [];
     	
     	--Elements	
     	patternTags<-singleListBox psub [tooltip:="Select the tag in which you want to insert the "++t];
     	name<-staticText psub [text:=t];
     	subT <- textEntry psub [text:=""];
     	bool <- radioBox psub Vertical ["And","Or","Xor"] [tooltip:="..."];
     	insertSubT <- button psub [text:="Insert"];
     	quit <- button psub [text:="Cancel"];
     	
     	--Layout
    	set wsub [layout := container psub (margin 5 $ fill $
    				row 5 [
    					fill $ widget patternTags,
    					column 5 [
    						row 5 [widget name, hfill $ widget subT],
    						row 5 [widget bool, column 3 [widget insertSubT, widget quit]]
    						 ]
    				      ]
    			)];
    	windowSetClientSize wsub (sz 240 70);
    	
    	showModal wsub (\stop -> do {
	    			set insertSubT [on command := do {
	    					term <- get subT text;
	    					i <- get patternTags selection;
	    					j<-get bool selection;
						b<-get bool (item j);
	    					if (term=="")
	    					then stop (Nothing)
	    					else stop (Just (term,i,b))
						}];
				set quit [on command := stop (Nothing)];
					})
	    	}

viewResultDialog::Window a -> String -> IO (Maybe a)
viewResultDialog w s = do {
	wres <- dialog w [text:="Result"];
     	pres <- panel wres [];
	
	--Elements
	labRes <- textCtrl pres [text:=s, wrap:=WrapLine];
	textCtrlSetEditable labRes False;
	esc <- button pres [text:="Quit"];
	save <- button pres [text:="Save XML"];
	
	--Layout
	set wres [layout := container pres (margin 5 $ fill $
				column 5 [
					fill $ boxed "Query Result" $ fill $ minsize (sz 300 150) $ widget labRes,
					floatCentre $ row 5 [widget save, widget esc]
					]
			)];
	
	set save [on command := do {
			fname<-fileSaveDialog wres False False "Save Result" [("XML File", ["*.xml"])] "C:'\'XMLFiltering" "";
			case fname of
				Nothing->warningDialog wres "Attention" "The path specified does not exist!"
				Just filepath -> appendFile filepath s
		}];
	
	showModal wres (\stop -> do {
		set esc [on command := stop(Nothing)];
			})
	}

extractQueryfromFile itemList [] q 
	|(null q) = itemAppend itemList "Empty File"
	|otherwise = itemAppend itemList ("Query: "++(reverse q))
extractQueryfromFile itemList (t:ts) q
	|t=='\n' = do { itemAppend itemList ("Query: "++(reverse q));
			extractQueryfromFile itemList ts []}
	|otherwise = extractQueryfromFile itemList ts (t:q)

addItems obj [v] = itemAppend obj v
addItems obj (v:vs) = do {
		itemAppend obj v;
		addItems obj vs}

controlVars::[String]->String->Bool
controlVars _ (x:y:xs) = False
controlVars [] [x] = isUpper x
controlVars (v:vs) x
	|v==x = False
	|otherwise = controlVars vs x

controlSon s "With renaming" = (('$':(s++"()")),(Tree (ELNode s [("type","N")]) []))
controlSon s "Without renaming" = ((s++"()"),(Tree (ELNode s []) []))

controlNumVar i tag
	|i>0 = 0
	|(tag == "Root") = 0
	|otherwise = 1

controlNumVar2 i j
	|i==0 = 0
	|j>=0 = 0
	|otherwise = 1

verifyButtonVar obj 0 = do {set obj[enabled:=False]}
verifyButtonVar obj 1 = return()